#include "cjxFireiCamera.h"


#define FI_STATUS_PRINT(Name, OnError, Func) \
printf("[CAM] %s ... \0", Name); \
fiStatus = Func; \
if (fiStatus == FIREi_STATUS_SUCCESS) printf("OK\n"); \
else {printf("ERROR\n -> %s\n", FiStatusString(fiStatus)); OnError;}


cjxFireiCamera::cjxFireiCamera()
{
    bThreadRunning = false;
    bIsFrameNew = false;
    bCameraOpen = false;
    uiQueuedFrames = 10;
    uiBufferBytes = 640*480;
    bBufferBlock = false;
    pBuffer = new unsigned char[uiBufferBytes];
    pPixels = new unsigned char[uiBufferBytes];

    fiIsochEngineExHandle = NULL; /// Must be set to NULL, otherwise there is a crash when initialized

    /// Check the static counter to see if Fi is already initialized
    /// Increase the counter
    FiInitialize();
}

cjxFireiCamera::~cjxFireiCamera()
{
    CloseCamera();

    /// Decrease a static counter of this class that keeps track of how many instances uses Fi.
    /// If the counter reaches 0, then terminate...
    printf("[CAM] FiTerminate ... \0");
    FiTerminate();
    printf("OK\n");

    delete pBuffer;
    delete pPixels;
}

bool
cjxFireiCamera::OpenCamera(int iDevice)
{
    if (bCameraOpen)
    {
        printf("[CAM] Can't open device %d, Another device is already open\n", iDevice);
        return false;
    }

//    FIREi_CAMERA_FRAME  CameraFrame;

    //ListDevices();
    if (!SetCameraHandle(iDevice))
        return false;

    if (!StartCamera())
        return false;

    if (!StartIsochEngine())
        return false;

    /// start thread
    bThreadContinue = true;
    _beginthread(GetFrameThread, 0, this);

    printf("[CAM] Thread started\n");
    bCameraOpen = true;
    return true;
}

bool
cjxFireiCamera::CloseCamera()
{
    FIREi_STATUS fiStatus;

    /// stop thread
    bThreadContinue = false;
    printf("[CAM] Waiting for thread to stop ...\n");
    while (bThreadRunning) {}
    printf("[CAM] OK\n");

    if (!StopCamera())
    {
        printf("[CAM] StopCamera failed\n");
        return false;
    }

    if (!StopIsochEngine())
    {
        printf("[CAM] StopIsochEngine failed\n");
        return false;
    }

    if (!FreeCameraHandle())
    {
        printf("[CAM] FreeCameraHandle failed\n");
        return false;
    }

    bCameraOpen = false;
    return true;
}

bool
cjxFireiCamera::ListDevices()
{
    ULONG               uCamerasFound = 10; /// max number to list
    FIREi_CAMERA_GUID   fiCameraGuid[uCamerasFound];
    FIREi_STATUS        fiStatus;

    memset(fiCameraGuid, 0, uCamerasFound * sizeof(FIREi_CAMERA_GUID));

    FI_STATUS_PRINT("FiLocateCameras", return false,
    FiLocateCameras(fiCameraGuid,FIREi_LOCATE_NEW_CAMERAS,&uCamerasFound);
    );

    printf("[CAM] ---- 1394 Devices ----\n");
    printf("[CAM] Found %d devices\n", uCamerasFound);
    for (int i = 0; i < uCamerasFound; ++i)
    {
    printf("[CAM] %d. Bytes:", i);
    for (int j = 0; j < 8; ++j) printf(" %d", fiCameraGuid[i].Bytes[j]);
    printf("\n");
    }
    printf("[CAM] ----------------------\n");

    return true;
}

bool
cjxFireiCamera::SetCameraHandle(int iDevice)
{
    ULONG               uCamerasFound = iDevice + 1;
    FIREi_CAMERA_GUID   fiCameraGuid[uCamerasFound];
    FIREi_STATUS        fiStatus;

    //ZeroMemory(&fiCameraGuid, sizeof(FIREi_CAMERA_GUID));
    memset(&fiCameraGuid, 0, sizeof(FIREi_CAMERA_GUID));

    FI_STATUS_PRINT("FiLocateCameras", return false,
    FiLocateCameras(fiCameraGuid,FIREi_LOCATE_NEW_CAMERAS,&uCamerasFound);
    );

    if (uCamerasFound <= iDevice)
    {
        printf("[CAM] Device %d not found (%d)\n", iDevice, uCamerasFound);
        return false;
    }

    FI_STATUS_PRINT("FiOpenCameraHandle", return false,
    FiOpenCameraHandle(&fiCameraHandle, &(fiCameraGuid[iDevice]));
    );

    //printf("[CAM] Connected first camera\n");
    bCameraHandleSet = true;
    return true;
}

bool
cjxFireiCamera::FreeCameraHandle()
{
    FIREi_STATUS        fiStatus;

    if(fiCameraHandle)
    {
        FI_STATUS_PRINT("FiCloseCameraHandle", return false,
        FiCloseCameraHandle(fiCameraHandle);
        );
    }

    bCameraHandleSet = false;
    return true;
}

bool
cjxFireiCamera::StartCamera()
{
    FIREi_STATUS fiStatus;

    fiStartupInfo.Tag           = FIREi_CAMERA_STARTUP_INFO_TAG;
    fiStartupInfo.FrameRate     = fps_30;
    fiStartupInfo.VideoMode     = Mode_5;
    fiStartupInfo.VideoFormat   = Format_0;
    fiStartupInfo.TransmitSpeed = S400;
    fiStartupInfo.IsochSyCode   = 1;
    fiStartupInfo.ChannelNumber = (UCHAR)0;

    // Start the camera
    FI_STATUS_PRINT("FiStartCamera", return false,
    FiStartCamera( fiCameraHandle, &fiStartupInfo);
    );

    printf("[CAM] Camera started\n");
    bCameraRunning = true;
    return true;
}

bool
cjxFireiCamera::StopCamera()
{
    FIREi_STATUS fiStatus;

    if (fiCameraHandle)
    {
        FI_STATUS_PRINT("FiStopCamera", return false,
        FiStopCamera(fiCameraHandle);
        );
    }

    bCameraRunning = false;
    return true;
}

bool
cjxFireiCamera::StartIsochEngine()
{
    FIREi_STATUS fiStatus;

    FI_STATUS_PRINT("FiCreateIsochReceiveEngineEx", return false,
    FiCreateIsochReceiveEngineEx(&fiIsochEngineExHandle, 1, &fiChannelStartProcessingEvent);
    );

    FI_STATUS_PRINT("FiAllocateFrames", return false,
    FiAllocateFrames(fiIsochEngineExHandle, &fiStartupInfo, uiQueuedFrames);
    );

    FI_STATUS_PRINT("FiStartIsochReceiveEngineEx", return false,
    FiStartIsochReceiveEngineEx(fiIsochEngineExHandle);
    );

    printf("[CAM] Isoch engine started\n");
    bIsochEngineRunning = true;
    return true;
}

bool
cjxFireiCamera::StopIsochEngine()
{
    FIREi_STATUS fiStatus;

    if (fiIsochEngineExHandle)
    {
        FI_STATUS_PRINT("FiStopIsochReceiveEngineEx", return false,
        FiStopIsochReceiveEngineEx(fiIsochEngineExHandle);
        );

        FI_STATUS_PRINT("FiDeallocateFrames", return false,
        FiDeallocateFrames(fiIsochEngineExHandle);
        );

        FI_STATUS_PRINT("FiDeleteIsochReceiveEngineEx", return false,
        FiDeleteIsochReceiveEngineEx(fiIsochEngineExHandle);
        );
    }

    bIsochEngineRunning = false;
    return true;
}

void
cjxFireiCamera::Update()
{
    if (!bCameraOpen)
        return;

#if CJX_FIREI_CAMERA_THREADED
    if (bIsBufferUpdatedByThread)
    {
//        if (!bBufferBlock)
//        {
//            bBufferBlock = true;
            memcpy(pPixels, pBuffer, uiBufferBytes);
            bIsBufferUpdatedByThread = false;
            bIsFrameNew = true;
//            bBufferBlock = false;
//        }
    }
#else
    PFIREi_CAMERA_FRAME     pCameraFrame;
    ULONG                   uCompleteFrames;
    ULONG                   I;
    PFIREi_CAMERA_FRAME     pFramesArray[QUEUED_FRAMES];

    uCompleteFrames = 0;
    for(;;)
    {
        pCameraFrame = FiGetNextCompleteFrameEx(g_hIsochEngineEx);

        if (NULL == pCameraFrame)
          break;

        pFramesArray[uCompleteFrames] = pCameraFrame;
        uCompleteFrames++;
    }

    if(0 == uCompleteFrames)
    {
        printf("Zero Complete Frames\n");
    }

    I=0;

    if (uCompleteFrames > 1)
    {
        printf("%u Frames recieved\n",uCompleteFrames);

        // Requeue all the frames except for the last one
        for(I=0;I<(uCompleteFrames - 1);I++)
            FiReleaseFrame(pFramesArray[I]);
    }

    if(FIREi_STATUS_SUCCESS == pFramesArray[I]->FireiStatus)
    {
        FiDisplayCameraFrame(g_hDisplay,pFramesArray[I]);

        // copy pixels to global buffer
        if (iBufferBytes >= pFramesArray[I]->uFrameBufferBytes)
            memcpy(pBuffer, pFramesArray[I]->pCameraFrameBuffer, pFramesArray[I]->uFrameBufferBytes);
    }
    else
    {
        printf("Failed to receive camera frame with error %s\n",FiStatusString(pFramesArray[I]->FireiStatus));

    }

    FiReleaseFrame(pFramesArray[I]);
#endif
}

bool
cjxFireiCamera::IsFrameNew()
{
    return bCameraOpen && bIsFrameNew;
}

unsigned char*
cjxFireiCamera::GetPixels()
{
    bIsFrameNew = false;
    return pPixels;
}

void
cjxFireiCamera::GetFrameThread(void* _pThis)
{
    printf("[THREAD] Started\n");
    cjxFireiCamera* pThis = (cjxFireiCamera*)_pThis;
    pThis->bThreadRunning = true;

    /// thread variables
    PFIREi_CAMERA_FRAME     pFrame;
    int                     i;
    int                     iFrames;
//    printf("Framebuffer size: %d\n", pThis->uiQueuedFrames);
    PFIREi_CAMERA_FRAME     pFrames[pThis->uiQueuedFrames];


    while (pThis->bThreadContinue)
    {
        /// Capture frames to buffer
//        printf("[THREAD] loop begin\n");
        iFrames = 0;
        for(;;)
        {
            pFrame = FiGetNextCompleteFrameEx(pThis->fiIsochEngineExHandle);
            if (pFrame == NULL)
                break;

            pFrames[iFrames] = pFrame;
            iFrames++;
        }
//        printf("[THREAD] Incomming frames: %d\n", iFrames);

        if (iFrames > 0)
        {
            /// release old frames from queue
            for(i = 0; i < (iFrames - 1); ++i)
            {
//                printf("[THREAD] Release frame %d\n", i);
                FiReleaseFrame(pFrames[i]);
            }


//            printf("[THREAD] Released %d dropped frames\n", i);

            /// copy the newest frame to class frame buffer
            if (pFrames[i]->FireiStatus == FIREi_STATUS_SUCCESS)
            {
                if (!pThis->bBufferBlock && (pFrames[i]->uFrameBufferBytes <= pThis->uiBufferBytes))
                {
                    pThis->bBufferBlock = true;
                    memcpy(pThis->pBuffer, pFrames[i]->pCameraFrameBuffer, pFrames[i]->uFrameBufferBytes);
                    pThis->bIsBufferUpdatedByThread = true;
                    pThis->bBufferBlock = false;
                }
                else
                {
                    printf("[THREAD] blocked or too many bytes (%d <= %d)\n", pFrames[i]->uFrameBufferBytes, pThis->uiBufferBytes);
                }
                //printf("[THREAD] frame received: %dx%d = %d bytes\n", pFramesArray[I]->uFrameWidth, pFramesArray[I]->uFrameHeight, pFramesArray[I]->uFrameBufferBytes);
                //printf("Frame received at time: %u\n",pFramesArray[I]->dwTimeStamp);
            }
            else
            {
                printf("[THREAD] Failed to receive camera frame: %s\n",FiStatusString(pFrames[i]->FireiStatus));
            }

            /// release the last frame in queue, now the queue is empty
            FiReleaseFrame(pFrames[i]);
        }
    }

    pThis->bThreadRunning = false;
    printf("[THREAD] Stopped\n");
    _endthread();
}

bool
cjxFireiCamera::SetProperty(FIREi_OID oid, unsigned short value)
{
    printf("[CAM] Set property %d -> %d\n", oid, value);

    FIREi_CAMERA_FEATURE_CONTROL_REGISTER reg;
    reg.Tag = FIREi_CAMERA_FEATURE_CONTROL_REGISTER_TAG;
    reg.bSetOnePush = FALSE;
    reg.bSetOn = TRUE;
    reg.bSetAuto = FALSE;
    reg.ushCurValue = value;

    FIREi_STATUS fiStatus;

    FI_STATUS_PRINT("FiSetCameraRegister", return false,
    FiSetCameraRegister(fiCameraHandle, oid, &reg, sizeof(FIREi_CAMERA_FEATURE_CONTROL_REGISTER));
    );

    return true;
}
