#ifndef CJX_FIREI_CAMERA_H
#define CJX_FIREI_CAMERA_H

#define CJX_FIREI_CAMERA_THREADED 1

#include <windows.h>
#include <process.h>    /* _beginthread, _endthread */
#include <stddef.h>
#include <stdlib.h>
#include <conio.h>
#include <stdio.h>
#include "Firei.h"
//#define QUEUED_FRAMES    10

#include "Tools/Tools.h"

class cjxFireiCamera
{
public:
    cjxFireiCamera();
    ~cjxFireiCamera();

    void            Update();
    bool            IsFrameNew();
    unsigned char*  GetPixels();

    void            SetShutter(unsigned short value)        {SetProperty(OID_AUTO_EXPOSURE_CONTROL, clamp_i(value, 0, 511));}
    void            SetGain(unsigned short value)           {SetProperty(OID_GAIN_CONTROL, clamp_i(value, 0, 255));}
    void            SetIris(unsigned short value)           {SetProperty(OID_IRIS_CONTROL, clamp_i(value, 0, 4));}
    void            SetBlackLevel(unsigned short value)     {SetProperty(OID_BRIGHTNESS_CONTROL, clamp_i(value, 128, 383));}
    void            SetSharpness(unsigned short value)      {SetProperty(OID_SHARPNESS_CONTROL, clamp_i(value, 0, 255));}
    void            SetGamma(unsigned short value)          {SetProperty(OID_GAMMA_CONTROL, clamp_i(value, 0, 1));}
    bool            SetProperty(FIREi_OID oid, unsigned short value);

    bool            ListDevices();
    bool            OpenCamera(int iDevice);
    bool            CloseCamera();

protected:
    bool            bCameraOpen;

    static void     GetFrameThread(void* _pThis);
    bool            bThreadContinue;
    bool            bThreadRunning;

    /// init
    bool            bCameraHandleSet;
    bool            SetCameraHandle(int iDevice);
    bool            FreeCameraHandle();

    bool            bCameraRunning;
    bool            StartCamera();
    bool            StopCamera();

    bool            bIsochEngineRunning;
    bool            StartIsochEngine();
    bool            StopIsochEngine();


    bool            bVerbose;

    bool            bIsFrameNew;
    bool            bIsBufferUpdatedByThread;                           /// write by thread
    bool            bBufferBlock;                                       /// read/write by thread
    unsigned int    uiBufferBytes;                                      /// read by thread
    unsigned char*  pBuffer;                                            /// write by thread

    unsigned char*  pPixels;

    FIREi_ISOCH_ENGINE_EX_HANDLE  fiIsochEngineExHandle;                /// read by thread
    FIREi_CAMERA_HANDLE           fiCameraHandle;
    FIREi_CAMERA_STARTUP_INFO     fiStartupInfo;
    HANDLE                        fiChannelStartProcessingEvent;        /// can be used to retreave new frames
    unsigned int                  uiQueuedFrames;
};

#endif
